/*
 *  search.c
 *  
 *
 *  Created by Ethan Phelps-Goodman on 7/9/05.
 *
 */
 
 

 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "search.h"
#include "array.h"
#include "unbalTree.h"
#include "avlTree.h"
#include "splayTree.h"




// the timing function from proj1
int CurrentTime(void) {
    struct timeval tv;
    int err;
    err = gettimeofday(&tv, NULL);
    assert(err == 0);
    return tv.tv_sec * 1000 + tv.tv_usec / 1000;
}



// a global variable storing the implementation
// type for this run
// Used in the wrapper functions below
int IMPLEMENTATION_TYPE = -1;



// Wrapper functions:
// each of these simply calls the corresponding
// method from the implementation specified
// at the command line

void* newDictionary() {
	switch ( IMPLEMENTATION_TYPE ) {
		case ARRAY:
			return new_array();
		case UNBALANCED_TREE:
			return new_unbal_tree();
		case AVL_TREE:
			return new_avl_tree();
		case SPLAY_TREE:
			return new_splay_tree();
	}
	
	fprintf(stderr, "Implementation not set!" );
	return (void*)0;
}


void insert( void* data, char* word, char* file ) {
	switch ( IMPLEMENTATION_TYPE ) {
		case ARRAY:
			insert_array( data, word, file );
			return;
		case UNBALANCED_TREE:
			insert_unbal_tree( data, word, file );
			return;
		case AVL_TREE:
			insert_avl_tree( data, word, file );
			return;
		case SPLAY_TREE:
			insert_splay_tree( data, word, file );
			return;
	}


	fprintf(stderr, "Implementation not set!" );
}


StringList* find( void* data, char* word ) {
	switch ( IMPLEMENTATION_TYPE ) {
		case ARRAY:
			return find_array( data, word );
		case UNBALANCED_TREE:
			return find_unbal_tree( data, word );
		case AVL_TREE:
			return find_avl_tree( data, word );
		case SPLAY_TREE:
			return find_splay_tree( data, word );
	}
	
	fprintf(stderr, "Implementation not set!" );
	return (void*)0;
}


// opens the file given by fileName,
// reads each word and inserts that word
// into data.
void parseFile( void* data, char* fileName ) {
	char input[256];
	FILE* fp = fopen( fileName, "r" );
	
	// read each word
	while ( ! feof( fp ) ) {
		fscanf( fp, "%*[^a-zA-Z]" );
		fscanf( fp, "%[a-zA-Z]", input );
		
		// add the word to the map
		insert( data, input, fileName );
	
	}

	fclose( fp );
}


// prints all the strings stored in the list
void printStringList( StringList* l ) {
	if ( l == 0 ) {
		printf( "empty\n" );
		return;
	}
	
	while ( l != 0 ) {
		printf( l->string );
		printf( " " );
		
		l = l->next;
	}
	
	printf( "\n" );
}


// reads a line of input from the command line
void readLine( char* input ) {
	char c;
	
	int i;
	// stop at newline or eof
	for ( i = 0; (c = fgetc(stdin)) != '\n' && ! feof(stdin); i++ ) {
		input[i] = c;
	}

	// null-terminate the string
	input[i]=0;
}


// reads the command line args,
// parses the files,
// and goes into a search loop
int main(int argc, char** argv) {
	int implementationType = -1;

	// command options
	if ( strcmp( argv[1], "-testUnbalTree" ) == 0 ) {
		test_unbal_tree();
		return;
	} else if ( strcmp( argv[1], "-testAVLTree" ) == 0 ) {
		test_avl_tree();
		return;
	} else if ( strcmp( argv[1], "-testSplayTree" ) == 0 ) {
		test_splay_tree();
		return;
	} else if ( strcmp( argv[1], "-testArray" ) == 0 ) {
		test_array();
		return;
	} else if ( strcmp( argv[1], "-array" ) == 0 ) {
		IMPLEMENTATION_TYPE = ARRAY;
	} else if ( strcmp( argv[1], "-unbalTree" ) == 0 ) {
		IMPLEMENTATION_TYPE = UNBALANCED_TREE;
	} else if ( strcmp( argv[1], "-avlTree" ) == 0 ) {
		IMPLEMENTATION_TYPE = AVL_TREE;
	} else if ( strcmp( argv[1], "-splayTree" ) == 0 ) {
		IMPLEMENTATION_TYPE = SPLAY_TREE;
	} else {
		// didn't match any valid command arg.
		fprintf(stderr, "Command argument not recognized.\n" );
		return;
	}


	void* data = newDictionary();

	// time the parsing
	int parseStart = CurrentTime();

	// parse each file given on command line
	int i;
	for ( i = 2; i < argc; i++ ) {
		printf( "Parsing " );
		printf( argv[i] );
		printf( "...\n" );
		parseFile( data, argv[i] );
	}
	
	printf( "Parsing took %dms.\n", CurrentTime() - parseStart );
	
	
	int lookupStart = CurrentTime();
	
	// loop until eof or '.'
	char input[256];
	readLine( input );
	while ( strcmp( input, "." ) != 0 && ! feof(stdin) ) {		
		printf( "Searching for %s...\n", input );
		printStringList( find( data, input ) );
	
		readLine(input);
	
	}
	
	printf( "Lookup took %dms.\n", CurrentTime() - lookupStart );
		
	
}


